@@ -68,6 +68,7 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "iba/stl_sa_priv.h"
 #include "iba/stl_sm_priv.h"
 
+
 static Status_t	sa_Trap_Forward(SubscriberKeyp, STL_NOTICE *);
 
 extern	uint64_t	topology_wakeup_time;
@@ -407,6 +408,15 @@ int sa_TrapNeedsLogging(Port_t *portp, uint8_t *trap_count)
 	return 0;
 }
 
+static
+void _copy_CapMaskTrap(Port_t **portp, STL_TRAP_CHANGE_CAPABILITY_DATA *ccTrapRef) {
+	(*portp)->portData->portInfo.CapabilityMask.AsReg32 = ccTrapRef->CapabilityMask.AsReg32;
+	(*portp)->portData->portInfo.CapabilityMask3.AsReg16 = ccTrapRef->CapabilityMask3.AsReg16;
+
+
+}
+
+
 /*
  * Used for forwarding traps that came from the outside.
  * In this case the caller pass the incoming mai packet
@@ -492,7 +502,7 @@ sa_Trap(Mai_t *maip) {
 
 	/* Send a TrapRepress to the sender after inserting our Mkey */
     BSWAPCOPY_STL_MKEY(&sm_config.mkey, stl_mai_get_mkey(maip));
-	(void)mai_reply(fd_async, maip);
+	(void)mai_reply(fd_async->fdMai, maip);
 
 	/* Look for subscribers */
 	(void)vs_lock(&saSubscribers.subsLock);
@@ -548,92 +558,92 @@ sa_Trap(Mai_t *maip) {
             IB_LOG_INFINI_INFO_FMT( "sa_Trap", 
                    "Received a PORT STATE CHANGE trap from %s, TID="FMT_U64, desc, tid);
 		}
-		/* Must tell sm_top to re-sweep fabric */
-		sm_discovery_needed("Port State Change Trap", notice.IssuerLID);
+		vs_rdlock(&old_topology_lock);
+		if(topology_passcount > 0) {
+			portp = sm_find_node_and_port_lid(&old_topology, notice.IssuerLID, &nodep);
+			if(sm_flap_report_port_change_trap(&sm_popo, nodep))
+				/* tell sm_top to re-sweep fabric unless trap was generated from flapping port */
+				sm_discovery_needed("Port State Change Trap", notice.IssuerLID);
+		} else {
+			sm_discovery_needed("Port State Change Trap", notice.IssuerLID);
+		}
+		vs_unlock(&old_topology_lock);
+
 	} else if (notice.Attributes.Generic.TrapNumber == MAD_SMT_CAPABILITYMASK_CHANGE) {
 		STL_TRAP_CHANGE_CAPABILITY_DATA ccTrap;
+		STL_TRAP_CHANGE_CAPABILITY_DATA ccTrapRef;
 		
 		BSWAPCOPY_STL_TRAP_CHANGE_CAPABILITY_DATA((STL_TRAP_CHANGE_CAPABILITY_DATA*)notice.Data, &ccTrap);
 		sm_get_lid_info(desc, notice.IssuerLID);
 
 		if (ccTrap.u.AsReg16 == 0) {
+
+			memcpy(&ccTrapRef, &ccTrap, sizeof(STL_TRAP_CHANGE_CAPABILITY_DATA));
+
 			/* Change fields are zero so one of the capability bits must
  			 * have changed. Compare the new CapabilityMask/Mask3 to the previous
 			 * CapabilityMask/Mask3 for this port to determine the appropriate
 			 * action to take*/
-			(void)vs_wrlock(&old_topology_lock);
+			vs_wrlock(&old_topology_lock);
 			portp = sm_find_node_and_port_lid(&old_topology, notice.IssuerLID, &nodep);
 			if (nodep && sm_valid_port(portp)){
 
-				STL_PORT_INFO portInfo;
-				Status_t status;
-
 				//determine which bits in Capability Mask/Mask3 changed
 				ccTrap.CapabilityMask.AsReg32 ^= portp->portData->portInfo.CapabilityMask.AsReg32;
 				ccTrap.CapabilityMask3.AsReg16 ^= portp->portData->portInfo.CapabilityMask3.AsReg16;
 
 
 				if (ccTrap.CapabilityMask.s.IsSM){ //node's SM status changed, will need to trigger a sweep
-					if (!portp->portData->portInfo.CapabilityMask.s.IsSM){
+					if (ccTrapRef.CapabilityMask.s.IsSM){
 						IB_LOG_INFINI_INFO_FMT(__func__, 
 								"Received an (IS_SM on) CAPABILITYMASK CHANGE [0x%.8X] trap from %s, TID="FMT_U64,
-								ccTrap.CapabilityMask.AsReg32, desc, tid);
-						sm_discovery_needed("Port CapabilityMask Change isSM on", STL_LID_RESERVED);
+								ccTrapRef.CapabilityMask.AsReg32, desc, tid);
+						if (notice.IssuerLID != sm_lid){
+							sm_discovery_needed("Port CapabilityMask Change isSM on", STL_LID_RESERVED);
+						}
 					}else{
 						IB_LOG_INFINI_INFO_FMT(__func__, 
 								"Received an (IS_SM off) CAPABILITYMASK CHANGE [0x%.8X] trap from %s, TID="FMT_U64,
-								ccTrap.CapabilityMask.AsReg32, desc, tid);
+								ccTrapRef.CapabilityMask.AsReg32, desc, tid);
 						if (notice.IssuerLID != sm_lid){	
 							sm_discovery_needed("Port CapabilityMask Change isSM off", STL_LID_RESERVED);
 						}
 					}
 				}
 
-				// Get fresh port info
-				SmpAddr_t addr = SMP_ADDR_CREATE_LR(sm_lid, portp->portData->lid);
-				status = SM_Get_PortInfo(fd_async_request, (1 << 24) | portp->index, &addr, &portInfo);
-				if (status != VSTATUS_OK){
-					IB_LOG_WARN_FMT(__func__, 
-									"Cannot get PORTINFO for %s, TID="FMT_U64
-									" status=%d", desc, tid, status);
-				}else{
-					portp->portData->portInfo = portInfo;
-
-					/****************LOG BITS THAT CHANGED******************/
-					//portp->portData->portInfo contains current information
-					if(ccTrap.CapabilityMask.s.IsAutomaticMigrationSupported){
-							IB_LOG_INFINI_INFO_FMT(__func__, 
-									"Received an (IS_AUTOMATIC_MIGRATION_SUPPORTED %s) CAPABILITYMASK CHANGE [0x%.8X] trap from %s, TID="FMT_U64,
-									(portp->portData->portInfo.CapabilityMask.s.IsAutomaticMigrationSupported?"on":"off"),
-									ccTrap.CapabilityMask.AsReg32, desc, tid);
-					}
-					if(ccTrap.CapabilityMask.s.IsConnectionManagementSupported){
-							IB_LOG_INFINI_INFO_FMT(__func__, 
-									"Received an (IS_CONNECTION_MANAGEMENT_SUPPORTED %s) CAPABILITYMASK CHANGE [0x%.8X] trap from %s, TID="FMT_U64, 
-									(portp->portData->portInfo.CapabilityMask.s.IsConnectionManagementSupported?"on":"off"), 
-									ccTrap.CapabilityMask.AsReg32, desc, tid);
-					}
-					if (ccTrap.CapabilityMask.s.IsDeviceManagementSupported){
-							IB_LOG_INFINI_INFO_FMT(__func__, 
-									"Received an (IS_DEVICE_MANAGEMENT_SUPPORTED %s) CAPABILITYMASK CHANGE [0x%.8X] trap from %s, TID="FMT_U64, 
-									(portp->portData->portInfo.CapabilityMask.s.IsDeviceManagementSupported?"on":"off"),
-									ccTrap.CapabilityMask.AsReg32, desc, tid);
-					}
-					if(ccTrap.CapabilityMask.s.IsVendorClassSupported){
-							IB_LOG_INFINI_INFO_FMT(__func__, 
-									"Received an (IS_VENDOR_CLASS_SUPPORTED %s) CAPABILITYMASK CHANGE [0x%.8X] trap from %s, TID="FMT_U64, 
-									(portp->portData->portInfo.CapabilityMask.s.IsVendorClassSupported?"on":"off"),
-									ccTrap.CapabilityMask.AsReg32, desc, tid);
-					}
-					if(ccTrap.CapabilityMask.s.IsCapabilityMaskNoticeSupported){
-							IB_LOG_INFINI_INFO_FMT(__func__, 
-									"Received an (IS_CAPABILITY_MASK_NOTICE_SUPPORTED %s) CAPABILITYMASK CHANGE [0x%.8X] trap from %s, TID="FMT_U64, 
-									(portp->portData->portInfo.CapabilityMask.s.IsCapabilityMaskNoticeSupported?"on":"off"),
-									ccTrap.CapabilityMask.AsReg32, desc, tid);
-					}
+				if(ccTrap.CapabilityMask.s.IsAutomaticMigrationSupported){
+					IB_LOG_INFINI_INFO_FMT(__func__,
+							"Received an (IS_AUTOMATIC_MIGRATION_SUPPORTED %s) CAPABILITYMASK CHANGE [0x%.8X] trap from %s, TID="FMT_U64,
+							(ccTrapRef.CapabilityMask.s.IsAutomaticMigrationSupported?"on":"off"),
+							ccTrapRef.CapabilityMask.AsReg32, desc, tid);
 				}
+				if(ccTrap.CapabilityMask.s.IsConnectionManagementSupported){
+					IB_LOG_INFINI_INFO_FMT(__func__,
+							"Received an (IS_CONNECTION_MANAGEMENT_SUPPORTED %s) CAPABILITYMASK CHANGE [0x%.8X] trap from %s, TID="FMT_U64, 
+							(ccTrapRef.CapabilityMask.s.IsConnectionManagementSupported?"on":"off"), 
+							ccTrapRef.CapabilityMask.AsReg32, desc, tid);
+				}
+				if (ccTrap.CapabilityMask.s.IsDeviceManagementSupported){
+					IB_LOG_INFINI_INFO_FMT(__func__,
+							"Received an (IS_DEVICE_MANAGEMENT_SUPPORTED %s) CAPABILITYMASK CHANGE [0x%.8X] trap from %s, TID="FMT_U64, 
+							(ccTrapRef.CapabilityMask.s.IsDeviceManagementSupported?"on":"off"),
+							ccTrapRef.CapabilityMask.AsReg32, desc, tid);
+				}
+				if(ccTrap.CapabilityMask.s.IsVendorClassSupported){
+					IB_LOG_INFINI_INFO_FMT(__func__,
+							"Received an (IS_VENDOR_CLASS_SUPPORTED %s) CAPABILITYMASK CHANGE [0x%.8X] trap from %s, TID="FMT_U64, 
+							(ccTrapRef.CapabilityMask.s.IsVendorClassSupported?"on":"off"),
+							ccTrapRef.CapabilityMask.AsReg32, desc, tid);
+				}
+				if(ccTrap.CapabilityMask.s.IsCapabilityMaskNoticeSupported){
+					IB_LOG_INFINI_INFO_FMT(__func__,
+							"Received an (IS_CAPABILITY_MASK_NOTICE_SUPPORTED %s) CAPABILITYMASK CHANGE [0x%.8X] trap from %s, TID="FMT_U64, 
+							(ccTrapRef.CapabilityMask.s.IsCapabilityMaskNoticeSupported?"on":"off"),
+							ccTrapRef.CapabilityMask.AsReg32, desc, tid);
+				}
+				_copy_CapMaskTrap(&portp, &ccTrapRef);
 			}
-			(void)vs_rwunlock(&old_topology_lock);
+			vs_rwunlock(&old_topology_lock);
 		} else {
 			/* A local change has occurred. */
 			IB_LOG_INFINI_INFO_FMT( "sa_Trap", 
@@ -844,7 +854,7 @@ static Status_t sa_Trap_Forward(SubscriberKeyp subsKeyp, STL_NOTICE * noticep) {
 
 	IB_ENTER("sa_Trap_Forward", subsKeyp, noticep, 0, 0);
 
-	(void)mai_alloc_tid(fd_saTrap, MAD_CV_SUBN_ADM, &tid);
+	(void)mai_alloc_tid(fd_saTrap->fdMai, MAD_CV_SUBN_ADM, &tid);
 
 	INCREMENT_COUNTER(smCounterSaTxReportNotice);
 
@@ -901,7 +911,7 @@ static Status_t sa_Trap_Forward(SubscriberKeyp subsKeyp, STL_NOTICE * noticep) {
     if ((madcntxt = cs_cntxt_get(&out_mad, &sm_notice_cntxt, FALSE)) == NULL) {
         // could not get a context, send report unreliably
         IB_LOG_WARN0("sa_Trap_Forward: can't allocate a notice context, sending unreliably");
-		if ((status = mai_send(fd_saTrap, &out_mad)) != VSTATUS_OK) {
+		if ((status = mai_send(fd_saTrap->fdMai, &out_mad)) != VSTATUS_OK) {
 			IB_LOG_ERRORRC("sa_Trap_Forward: can't send MAD unreliably rc:", status);
 		}
     } else {
