@@ -47,6 +47,13 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "iba/public/datatypes.h"
 #include "iba/public/ibyteswap_osd.h"
 
+#if defined(__LINUX__) && defined(_BSD_SOURCE)
+#include <endian.h>
+#elif defined(VXWORKS)
+#include <../src/hwif/h/vxbus/vxbAccess.h>
+#endif
+
+
 #ifdef __cplusplus
 extern "C"
 {
@@ -74,6 +81,13 @@ extern "C"
 	#endif
 	#define hton16				ntoh16
 #endif
+#ifndef le16toh
+	#if CPU_BE
+		#define le16toh( x )	( (((x) & 0x00FF) << 8) | (((x) & 0xFF00) >> 8) )
+	#else
+		#define le16toh
+	#endif
+#endif
 
 /* Default implementation of 32 bit swap function. */
 #ifndef ntoh32
@@ -87,6 +101,16 @@ extern "C"
 	#endif
 #define hton32				ntoh32
 #endif
+#ifndef le32toh
+	#if CPU_BE
+		#define le32toh( x ) ( (((x) & 0x000000FF) << 24) |	\
+								(((x) & 0x0000FF00) << 8) |	\
+								(((x) & 0x00FF0000) >> 8) |	\
+								(((x) & 0xFF000000) >> 24) )
+	#else
+		#define le32toh
+	#endif
+#endif
 
 /* Default implementation of 64 bit swap function. */
 #ifndef ntoh64
@@ -104,6 +128,20 @@ extern "C"
 	#endif
 #define hton64				ntoh64
 #endif
+#ifndef le64toh
+	#if CPU_BE
+		#define le64toh( x )	( (((x) & 0x00000000000000FF) << 56)  |		\
+								(((x) & 0x000000000000FF00) << 40)  |	\
+								(((x) & 0x0000000000FF0000) << 24)  |	\
+								(((x) & 0x00000000FF000000) << 8 )  |	\
+								(((x) & 0x000000FF00000000) >> 8 )  |	\
+								(((x) & 0x0000FF0000000000) >> 24)  |	\
+								(((x) & 0x00FF000000000000) >> 40)  |	\
+								(((x) & 0xFF00000000000000) >> 56) )
+	#else
+		#define le64toh
+	#endif
+#endif
 
 
 /* 
